@@ -36,18 +36,20 @@
  *
  */
 #include <stdarg.h>
-#include <libgnomeui/libgnomeui.h>
+/*#include <libgnomeui/libgnomeui.h>*/
+#include <string.h>
 #include <eel/eel-gconf-extensions.h>
 #include <mlview-marshal.h>
 #include "mlview-app-context.h"
 #include "mlview-file-descriptor.h"
 #include "mlview-utils.h"
+#include "mlview-xml-document.h"
 
-
-/**********************************
+/*********************************
  *Gconf keys used throughout the
  *application:
  *********************************/
+
 #define DEFAULT_TREE_EXPANSION_DEPTH_KEY "/apps/mlview/default-tree-expansion-depth"
 #define VALIDATION_IS_ON_KEY "/apps/mlview/validation-is-on"
 #define DEFAULT_EDITING_VIEW_TYPE_KEY "/apps/mlview/default-editing-view-type"
@@ -62,13 +64,26 @@ static const gchar *tree_editors_node_colours_keys[NB_OF_TREE_NODE_COLOURS + 1]
  "/apps/mlview/colours/xml-pi-node",
  "/apps/mlview/colours/xml-dtd-node",
  "/apps/mlview/colours/xml-entity-decl-node",
- 
  NULL} ;
 
+/**
+ *Some icons.
+ */
+#define ELEMENT_ICON      "mlview/xml-element-node.png"
+#define OPEN_ELEMENT_ICON "mlview/xml-element-node-open.png"
+#define TEXT_ICON         "mlview/xml-text-node.png"
+#define ROOT_ICON         "mlview/xml-root.png"
+#define OPEN_ROOT_ICON    "mlview/xml-root-open.png"
+#define COMMENT_ICON      "mlview/xml-comment-node.png"
+#define PI_ICON           "mlview/xml-pi-node.png"
+#define ENTITY_REF_ICON   "mlview/xml-entity-ref-node.png"
+
 enum NumberOfSignals {
         APPLICATION_INITIALIZED,
         CONTEXTUAL_MENU_REQUESTED,
         VIEW_SWAPPED,
+        DOCUMENT_NAME_CHANGED,
+        DOCUMENT_UNDO_STATE_CHANGED,
         NUMBER_OF_SIGNALS
 } ;
 
@@ -119,6 +134,13 @@ struct _MlViewAppContextPrivate {
 	 */
         MlViewFileSelection *file_sel;
 
+        /**
+         *A cache for the instance of GtkFileChooser (for gtk+ >=2.4)
+         *components should use this instance of file chooser.         
+         */
+#ifdef MLVIEW_WITH_GTK_FILE_CHOOSER
+        GtkFileChooser *file_chooser ;
+#endif
         /**
 	 *The xml catalog loaded by mlview
 	 */
@@ -135,6 +157,17 @@ struct _MlViewAppContextPrivate {
          */
         GConfClient * gconf_client ;
         gboolean dispose_has_run;
+
+        /**
+         *the refcount on 
+         *the type icons
+         */
+        guint type_icons_refcnt;
+
+        /**
+         *the type icons
+         */
+        struct MlViewTypeIcons *type_icons;
 };
 
 
@@ -146,7 +179,7 @@ struct _MlViewAppContextPrivate {
 static GObjectClass *gv_parent_class = NULL;
 
 static void
-mlview_app_context_init_class (MlViewAppContextClass * a_klass);
+mlview_app_context_class_init (MlViewAppContextClass * a_klass);
 
 static void
 mlview_app_context_init (MlViewAppContext * a_ctxt);
@@ -187,17 +220,17 @@ static gboolean is_a_gconf_entry_a_tree_editor_node_color (GConfEntry *a_entry,
  *@return the id of the type of
  *#MlViewAppContext.
  */
-guint
+GType
 mlview_app_context_get_type (void)
 {
-        static guint type = 0;
+        static GType type = 0;
 
         if (!type) {
                 static const GTypeInfo type_info = {
                         sizeof (MlViewAppContextClass),
                         NULL, NULL,
                         (GClassInitFunc)
-                                mlview_app_context_init_class,
+                                mlview_app_context_class_init,
                         NULL, NULL,
                         sizeof (MlViewAppContext),
                         0,
@@ -216,7 +249,7 @@ mlview_app_context_get_type (void)
  *@param a_klass the vtable structure of MlViewAppContext.
  */
 static void
-mlview_app_context_init_class (MlViewAppContextClass * a_klass)
+mlview_app_context_class_init (MlViewAppContextClass * a_klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (a_klass);
 
@@ -254,10 +287,30 @@ mlview_app_context_init_class (MlViewAppContextClass * a_klass)
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET
                               (MlViewAppContextClass,
-                               view_changed), NULL, NULL,
+                               view_swapped), NULL, NULL,
                               mlview_marshal_VOID__POINTER_POINTER,
                               G_TYPE_NONE, 2, G_TYPE_POINTER,
                               G_TYPE_POINTER);
+        
+        gv_signals[DOCUMENT_NAME_CHANGED] =
+                g_signal_new ("document-name-changed",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET
+                              (MlViewAppContextClass, document_name_changed), 
+                              NULL, NULL,
+                              mlview_marshal_VOID__POINTER,
+                              G_TYPE_NONE, 1, G_TYPE_POINTER);
+
+        gv_signals[DOCUMENT_UNDO_STATE_CHANGED] =
+                g_signal_new ("document-undo-state-changed",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET
+                              (MlViewAppContextClass,
+                               document_undo_state_changed), NULL, NULL,
+                              mlview_marshal_VOID__POINTER,
+                              G_TYPE_NONE, 1, G_TYPE_POINTER);
 }
 
 
@@ -311,7 +364,13 @@ mlview_app_context_dispose (GObject * a_object)
                                     (PRIVATE (ctxt)->file_sel));
                 PRIVATE (ctxt)->file_sel = NULL;
         }
-
+#ifdef MLVIEW_WITH_GTK_FILE_CHOOSER
+        if (PRIVATE (ctxt)->file_chooser) {
+                gtk_widget_destroy (GTK_WIDGET 
+                                    (PRIVATE (ctxt)->file_chooser)) ;
+                PRIVATE (ctxt)->file_chooser = NULL ;
+        }
+#endif
         if (PRIVATE (ctxt)->bitmaps_cache) {
                 g_hash_table_foreach (PRIVATE (ctxt)->
                                       bitmaps_cache, (GHFunc)
@@ -656,6 +715,48 @@ mlview_app_context_notify_application_initialized (MlViewAppContext *a_this)
         return MLVIEW_OK ;
 }
 
+/**
+ *Notifies listeners that the name of the document being edited has
+ *changed.
+ *@param a_this the current instance of #MlViewAppContext
+ *@param a_doc the document which name has changed.
+ *@return MLVIEW_OK upon successful completion.
+ */
+enum MlViewStatus 
+mlview_app_context_notify_document_name_changed (MlViewAppContext *a_this,
+                                                 gpointer a_doc)
+{
+        g_return_val_if_fail (a_this && MLVIEW_IS_APP_CONTEXT (a_this)
+                              && PRIVATE (a_this),
+                              MLVIEW_BAD_PARAM_ERROR) ;
+        g_return_val_if_fail (a_doc && MLVIEW_IS_XML_DOCUMENT (a_doc),
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        if (a_doc) {
+                g_signal_emit (G_OBJECT (a_this),
+                               gv_signals[DOCUMENT_NAME_CHANGED], 0,
+                               a_doc) ;
+        }
+        return MLVIEW_OK ;
+}
+
+
+enum MlViewStatus 
+mlview_app_context_notify_document_undo_state_changed (MlViewAppContext *a_this,
+                                                       gpointer  a_doc)
+{
+        g_return_val_if_fail (a_this && MLVIEW_IS_APP_CONTEXT (a_this)
+                              && PRIVATE (a_this),
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        if (a_doc) {
+                g_signal_emit (G_OBJECT (a_this),
+                               gv_signals[DOCUMENT_UNDO_STATE_CHANGED], 0,
+                               a_doc) ;
+        }
+        return MLVIEW_OK ;
+}
+
 /**
  *Pushes a message on the app bar. 
  *@param a_context the current instance of MlViewAppContext.
@@ -688,19 +789,16 @@ mlview_app_context_sbar_push_message (MlViewAppContext *a_context,
  *The effect is that the message pushed before the 
  *last message has been pushed, will be
  *displayed. 
- *
  *@param a_context the current instance of MlViewAppContext.
- *
  */
 void
 mlview_app_context_sbar_pop_message (MlViewAppContext *
                                      a_context)
 {
-        
         /*
          *FIXME: this is broken since I dumped
          *the GnomeApp stuffs.
-         */       
+         */
 }
 
 
@@ -1245,32 +1343,49 @@ mlview_app_context_get_file_selector (MlViewAppContext *a_this,
 }
 
 
-/**
- *Makes a_window always stay on top of the mlview application root window. 
- *
- *@param a_window the window that must be transient for 
- *the mlview app root window.
- *@param a_this the current application context.
- */
-void
-mlview_app_context_set_window_transient_for_app (MlViewAppContext* a_this,
-                                                 GtkWindow * a_window)
+#ifdef MLVIEW_WITH_GTK_FILE_CHOOSER
+GtkFileChooser *
+mlview_app_context_get_file_chooser (MlViewAppContext *a_this,
+                                     const gchar *a_title)
 {
-        GnomeApp *app = NULL;
+        g_return_val_if_fail (a_this 
+                              && MLVIEW_IS_APP_CONTEXT (a_this)
+                              && PRIVATE (a_this),
+                              NULL) ;
 
-        g_return_if_fail (a_this != NULL);
-        g_return_if_fail (MLVIEW_IS_APP_CONTEXT (a_this));
-        g_return_if_fail (a_window != NULL);
-        g_return_if_fail (GTK_IS_WINDOW (a_window));
-
-        app = mlview_app_context_get_element (a_this,
-                                              "GnomeApp");
-        g_return_if_fail (app != NULL);
-        g_return_if_fail (GNOME_IS_APP (app));
-
-        gtk_window_set_transient_for (a_window,
-                                      GTK_WINDOW (app));
+        if (!PRIVATE (a_this)->file_chooser) {
+                GtkWidget * parent_window = NULL ;
+                parent_window = mlview_app_context_get_element (a_this,
+                                                                "MlViewMainWindow") ;
+                PRIVATE (a_this)->file_chooser = GTK_FILE_CHOOSER (gtk_file_chooser_dialog_new
+                                                                   (a_title,
+                                                                    GTK_WINDOW (parent_window),
+                                                                    GTK_FILE_CHOOSER_ACTION_SAVE, 
+                                                                    GTK_STOCK_CANCEL, 
+                                                                    GTK_RESPONSE_CANCEL, 
+                                                                    GTK_STOCK_OPEN,
+                                                                    GTK_RESPONSE_OK,
+                                                                    NULL));
+                g_return_val_if_fail (PRIVATE (a_this)->file_chooser != NULL, NULL) ;
+                g_signal_connect (G_OBJECT (PRIVATE (a_this)->file_chooser),
+                                  "delete-event",
+                                  G_CALLBACK (gtk_widget_hide_on_delete),
+                                  NULL) ;
+                gtk_window_set_modal (GTK_WINDOW (PRIVATE (a_this)->file_chooser),
+                                      TRUE) ;
+                gtk_file_chooser_set_local_only 
+                        (PRIVATE (a_this)->file_chooser, TRUE);
+                gtk_file_chooser_set_select_multiple 
+                        (GTK_FILE_CHOOSER (PRIVATE (a_this)->file_chooser),
+                         FALSE);
+        }
+        g_return_val_if_fail (PRIVATE (a_this)->file_chooser != NULL, NULL) ;
+        gtk_window_set_title (GTK_WINDOW (PRIVATE (a_this)->file_chooser),
+                              a_title) ;
+        
+        return PRIVATE (a_this)->file_chooser ;
 }
+#endif
 
 /**
  *Associates an xml catalog to the current instance of MlViewAppContext.
@@ -1290,6 +1405,218 @@ mlview_app_context_set_xml_catalog (MlViewAppContext *a_this,
         PRIVATE (a_this)->xml_catalog = a_xml_catalog;
 }
 
+/**
+ *Unloads the type icons used to represent different types
+ *of XML nodes in some components of mlview.
+ *@param a_this the current instance of MlViewAppContext.
+ */
+static void
+mlview_app_context_unload_type_icons (MlViewAppContext *a_this)
+{
+        struct MlViewTypeIcons *icons = NULL;
+
+        g_return_if_fail (a_this && MLVIEW_IS_APP_CONTEXT (a_this));
+        g_return_if_fail (PRIVATE (a_this));
+
+        icons = PRIVATE (a_this)->type_icons;
+
+        if (icons) {
+                if (icons->element) {
+                        g_object_unref (G_OBJECT (icons->element));
+                        icons->element = NULL ;
+                }
+                if (icons->open_element) {
+                        g_object_unref (G_OBJECT (icons->open_element));
+                        icons->open_element = NULL ;
+                }
+                if (icons->text) {
+                        g_object_unref (G_OBJECT (icons->text));
+                        icons->text = NULL ;
+                }
+                if (icons->root) {
+                        g_object_unref (G_OBJECT (icons->root));
+                        icons->root = NULL ;
+                }
+                if (icons->open_root) {
+                        g_object_unref (G_OBJECT (icons->open_root));
+                        icons->open_root = NULL ;
+                }
+                if (icons->comment) {
+                        g_object_unref (G_OBJECT (icons->comment));
+                        icons->comment = NULL ;
+                }
+                if (icons->pi) {
+                        g_object_unref (G_OBJECT (icons->pi));
+                        icons->pi = NULL ;
+                }
+                if (icons->entity_ref) {
+                        g_object_unref (G_OBJECT (icons->entity_ref));
+                        icons->entity_ref = NULL ;
+                }
+
+                g_free (icons);
+
+                icons = NULL;
+                PRIVATE (a_this)->type_icons = NULL;
+        }
+}
+
+/**
+ *Loads the type icons used to represent different types 
+ *of XML nodes in some components of mlview.
+ *@param a_this the current instance of MlViewAppContext.
+ *@return a MlViewTypeIcons struct which contains the icons.
+ */
+static struct MlViewTypeIcons *
+mlview_app_context_load_type_icons (MlViewAppContext *a_this)
+{
+        gchar *path = NULL;
+        struct MlViewTypeIcons *type_icons = NULL;
+
+        g_return_val_if_fail (a_this && MLVIEW_IS_APP_CONTEXT (a_this), NULL);
+        g_return_val_if_fail (PRIVATE (a_this), NULL);
+
+        type_icons = g_try_malloc (sizeof (struct MlViewTypeIcons));
+
+        if (!type_icons) {
+                mlview_utils_trace_info ("malloc failed, system may be out of memory");
+
+                return NULL;
+        }
+
+        path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+                                          ELEMENT_ICON, TRUE, NULL);
+
+        if (!path) {
+                mlview_utils_trace_info ("lookup of icon file failed:") ;
+                mlview_utils_trace_info (ELEMENT_ICON) ;
+        } else {
+                type_icons->element = gdk_pixbuf_new_from_file (path, NULL);
+                g_free (path);
+        }
+
+        path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+                                          OPEN_ELEMENT_ICON, TRUE, NULL);
+        if (!path) {
+                mlview_utils_trace_info ("lookup of icon file failed:") ;
+                mlview_utils_trace_info (OPEN_ELEMENT_ICON) ;
+        } else {
+                type_icons->open_element = gdk_pixbuf_new_from_file (path, NULL);
+                g_free (path);
+        }
+
+        path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+                                          TEXT_ICON, TRUE, NULL);
+        if (!path) {
+                mlview_utils_trace_info ("lookup of icon file failed:") ;
+                mlview_utils_trace_info (TEXT_ICON) ;
+        } else {
+                type_icons->text = gdk_pixbuf_new_from_file (path, NULL);
+                g_free (path);
+        }
+
+        path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+                                          ROOT_ICON, TRUE, NULL);
+        if (!path) {
+                mlview_utils_trace_info ("lookup of icon file failed:") ;
+                mlview_utils_trace_info (ROOT_ICON) ;
+        } else {
+                type_icons->root = gdk_pixbuf_new_from_file (path, NULL);
+                g_free (path);
+        }
+
+        path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+                                          OPEN_ROOT_ICON, TRUE, NULL) ;
+        if (!path) {
+                mlview_utils_trace_info ("lookup of icon file failed:") ;
+                mlview_utils_trace_info (OPEN_ROOT_ICON) ;
+        } else {
+                type_icons->open_root = gdk_pixbuf_new_from_file (path, NULL);
+                g_free (path);
+        }
+
+        path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+                                          COMMENT_ICON, TRUE, NULL);
+        if (!path) {
+                mlview_utils_trace_info ("lookup of icon file failed:") ;
+                mlview_utils_trace_info (COMMENT_ICON) ;
+        } else {
+                type_icons->comment = gdk_pixbuf_new_from_file (path, NULL);
+                g_free (path);
+        }
+  
+        path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+                                          ENTITY_REF_ICON, TRUE, NULL);
+        if (!path) {
+                mlview_utils_trace_info ("lookup of icon file failed:") ;
+                mlview_utils_trace_info (ENTITY_REF_ICON) ;
+        } else {
+                type_icons->entity_ref = gdk_pixbuf_new_from_file (path, NULL);
+                g_free (path);
+        }
+
+        path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+                                          PI_ICON, TRUE, NULL);
+        if (!path) {
+                mlview_utils_trace_info ("lookup of icon file failed:") ;
+                mlview_utils_trace_info (PI_ICON) ;
+        } else {
+                type_icons->pi = gdk_pixbuf_new_from_file (path, NULL);
+                g_free (path);
+        }
+
+        return type_icons;
+}
+
+/**
+ *Decrements the reference count of the structure which
+ *countains the type icons used to represent different types
+ *of XML nodes used in some components of mlview.
+ *@param a_this the current instance of MlViewAppContext.
+ */
+void
+mlview_app_context_type_icons_unref (MlViewAppContext *a_this)
+{
+        g_return_if_fail (a_this && MLVIEW_IS_APP_CONTEXT (a_this));
+        g_return_if_fail (PRIVATE (a_this));
+
+        g_return_if_fail (PRIVATE (a_this)->type_icons_refcnt);
+
+        PRIVATE (a_this)->type_icons_refcnt--;
+        
+        if (!PRIVATE (a_this)->type_icons_refcnt) {
+                mlview_app_context_unload_type_icons (a_this);
+        }
+}
+
+/**
+ *Returns the type icons used to represent different types
+ *of XML nodes in some components of mlview.
+ *@param a_this the current instance of MlViewAppContext.
+ *@return a MlViewTypeIcons struct which contains the icons.
+ */
+struct MlViewTypeIcons *
+mlview_app_context_type_icons_ref (MlViewAppContext *a_this)
+{
+        g_return_val_if_fail (a_this && MLVIEW_IS_APP_CONTEXT (a_this), NULL);
+        g_return_val_if_fail (PRIVATE (a_this), NULL);
+
+        if (PRIVATE (a_this)->type_icons_refcnt) {
+                g_return_val_if_fail (PRIVATE (a_this)->type_icons, NULL);
+
+                PRIVATE (a_this)->type_icons_refcnt++;
+        } else {
+                g_return_val_if_fail (!PRIVATE (a_this)->type_icons, NULL);
+                
+                PRIVATE (a_this)->type_icons = mlview_app_context_load_type_icons (a_this);
+
+                g_return_val_if_fail (PRIVATE (a_this)->type_icons, NULL);
+
+                PRIVATE (a_this)->type_icons_refcnt++;
+        }
+
+        return PRIVATE (a_this)->type_icons;
+}
 
 /**
  *Getter of the xml catalog associated to the current instance of
