@@ -83,8 +83,18 @@ struct _MlViewIView {
         enum MlViewStatus (*update_contextual_menu) (MlViewIView *a_this,
                                                      GtkWidget **a_menu) ;
 
-	/*signals*/
-	void (*name_changed) (MlViewIView *a_this) ;
+        
+	/********************
+         *signals
+         ********************/
+
+	void (*name_changed) (MlViewIView *a_this, gpointer a_data) ;
+
+        void (*is_swapped_out)  (MlViewIView *a_this, gpointer a_data);
+
+        void  (*is_swapped_in) (MlViewIView *a_this, gpointer a_data) ;
+
+        void (*application_menu_populating_requested) (MlViewIView *a_this, gpointer a_data) ;
 };
 
 GType mlview_iview_get_type (void) ;
@@ -117,5 +127,11 @@ enum MlViewStatus mlview_iview_update_contextual_menu (MlViewIView *a_this,
 
 enum MlViewStatus mlview_iview_get_impl (MlViewIView *a_this, GtkWidget **a_impl) ;
 
+enum MlViewStatus mlview_iview_notify_swapped_out (MlViewIView *a_this) ;
+
+enum MlViewStatus mlview_iview_notify_swapped_in (MlViewIView *a_this);
+
+enum MlViewStatus mlview_iview_request_application_menu_populating (MlViewIView *a_this) ;
+
 G_END_DECLS
 #endif
