@@ -27,7 +27,7 @@
 #include <gnome.h>
 #include <glade/glade.h>
 #include "mlview-styled-view.h"
-#include "mlview-tree-editor2.h"
+#include "mlview-tree-editor.h"
 
 /**
  *Implementation of the #MlViewStyledView class.
@@ -38,7 +38,7 @@
 
 struct _MlViewStyledViewPriv {	
 	GtkPaned *main_paned ;
-	MlViewTreeEditor2 *tree_editor ;
+	MlViewTreeEditor *tree_editor ;
 	SXBoxView * styled_editor ;
 	gboolean dispose_has_run ;
         gboolean connected_to_doc ;
@@ -874,6 +874,7 @@ mlview_styled_view_construct (MlViewStyledView *a_this,
         CRStyleSheet *sheet = NULL ;
         CRCascade * cascade = NULL ;
         xmlDoc *xml_doc = NULL ;
+        GtkWidget *scroll = NULL ;
 
 	if (PRIVATE (a_this)->main_paned 
 	    || PRIVATE (a_this)->tree_editor
@@ -886,7 +887,7 @@ mlview_styled_view_construct (MlViewStyledView *a_this,
         g_return_val_if_fail (status == MLVIEW_VIEW_ADAPTER_OK,
                               status) ;
 
-        cr_status = sx_box_view_get_default_stylesheet (&sheet) ;
+        cr_status = sx_box_view_get_default_stylesheet (&sheet, FALSE) ;
         if (cr_status != CR_OK || ! sheet)
                 return MLVIEW_ERROR ;
         cascade = cr_cascade_new (NULL, NULL, sheet) ;
@@ -908,9 +909,9 @@ mlview_styled_view_construct (MlViewStyledView *a_this,
                 status = MLVIEW_OK ;
                 goto cleanup ;
         }
-        PRIVATE (a_this)->tree_editor = MLVIEW_TREE_EDITOR2
-                (mlview_tree_editor2_new (a_app_context)) ;
-        mlview_tree_editor2_edit_xml_doc (PRIVATE (a_this)->tree_editor,
+        PRIVATE (a_this)->tree_editor = MLVIEW_TREE_EDITOR
+                (mlview_tree_editor_new (a_app_context)) ;
+        mlview_tree_editor_edit_xml_doc (PRIVATE (a_this)->tree_editor,
                                           a_doc, (guchar*)"Experimental") ;
         PRIVATE (a_this)->main_paned = GTK_PANED (gtk_hpaned_new ()) ;
         gtk_box_pack_start (GTK_BOX (a_this), 
@@ -918,8 +919,10 @@ mlview_styled_view_construct (MlViewStyledView *a_this,
                             TRUE, TRUE, 0) ;
         gtk_paned_add1 (GTK_PANED (PRIVATE (a_this)->main_paned),
                          GTK_WIDGET (PRIVATE (a_this)->tree_editor)) ;
-        gtk_paned_add2 (GTK_PANED (PRIVATE (a_this)->main_paned),
-                        GTK_WIDGET (PRIVATE (a_this)->styled_editor)) ;
+        scroll = gtk_scrolled_window_new (NULL, NULL) ;
+        gtk_container_add (GTK_CONTAINER (scroll), 
+                           GTK_WIDGET (PRIVATE (a_this)->styled_editor)) ;
+        gtk_paned_add2 (GTK_PANED (PRIVATE (a_this)->main_paned), scroll) ;
         PRIVATE (a_this)->xml_doc = a_doc ;
         mlview_xml_document_ref (a_doc) ;
         gtk_widget_show_all (GTK_WIDGET (PRIVATE (a_this)->main_paned)) ;
@@ -1002,7 +1005,7 @@ mlview_styled_view_create_internal_subset_node_interactive (MlViewStyledView *a_
 enum MlViewStatus
 mlview_styled_view_add_child_node_interactive (MlViewStyledView *a_this)
 {
-        MlViewTreeEditor2 *tree_editor = NULL ;
+        MlViewTreeEditor *tree_editor = NULL ;
         
         g_return_val_if_fail (a_this && MLVIEW_IS_STYLED_VIEW (a_this)
                               && PRIVATE (a_this)
@@ -1011,7 +1014,7 @@ mlview_styled_view_add_child_node_interactive (MlViewStyledView *a_this)
 
         tree_editor = mlview_styled_view_get_tree_editor (a_this) ;
         g_return_val_if_fail (tree_editor, MLVIEW_ERROR) ;
-        mlview_tree_editor2_add_child_node_interactive (tree_editor) ;
+        mlview_tree_editor_add_child_node_interactive (tree_editor) ;
         return MLVIEW_OK ;
 }
 
@@ -1024,14 +1027,14 @@ mlview_styled_view_add_child_node_interactive (MlViewStyledView *a_this)
 enum MlViewStatus
 mlview_styled_view_insert_prev_sibling_node_interactive (MlViewStyledView *a_this)
 {
-        MlViewTreeEditor2 *tree_editor = NULL ;
+        MlViewTreeEditor *tree_editor = NULL ;
 
         g_return_val_if_fail (a_this && MLVIEW_IS_STYLED_VIEW (a_this),
                               MLVIEW_BAD_PARAM_ERROR) ;
 
         tree_editor = mlview_styled_view_get_tree_editor (a_this) ;
         g_return_val_if_fail (tree_editor, MLVIEW_ERROR) ;
-        mlview_tree_editor2_insert_prev_sibling_node_interactive (tree_editor) ;
+        mlview_tree_editor_insert_prev_sibling_node_interactive (tree_editor) ;
         return MLVIEW_OK ;
 }
 
@@ -1044,7 +1047,7 @@ mlview_styled_view_insert_prev_sibling_node_interactive (MlViewStyledView *a_thi
 enum MlViewStatus
 mlview_styled_view_insert_next_sibling_node_interactive (MlViewStyledView * a_this) 
 {
-        MlViewTreeEditor2 *tree_editor = NULL;
+        MlViewTreeEditor *tree_editor = NULL;
 
         g_return_val_if_fail (a_this && MLVIEW_IS_STYLED_VIEW (a_this),
                               MLVIEW_BAD_PARAM_ERROR) ;
@@ -1053,7 +1056,7 @@ mlview_styled_view_insert_next_sibling_node_interactive (MlViewStyledView * a_th
                 (a_this);
 
         if (tree_editor) {
-                mlview_tree_editor2_insert_next_sibling_node_interactive
+                mlview_tree_editor_insert_next_sibling_node_interactive
                         (tree_editor);
         }
         return MLVIEW_OK ;
@@ -1068,7 +1071,7 @@ mlview_styled_view_insert_next_sibling_node_interactive (MlViewStyledView * a_th
 enum MlViewStatus
 mlview_styled_view_cut_node (MlViewStyledView *a_this)
 {
-        MlViewTreeEditor2 *tree_editor = NULL;
+        MlViewTreeEditor *tree_editor = NULL;
         GtkTreeIter cur_sel_start = {0};
         enum MlViewStatus status = MLVIEW_OK ;
 
@@ -1077,10 +1080,10 @@ mlview_styled_view_cut_node (MlViewStyledView *a_this)
 
         if (tree_editor == NULL)
                 return MLVIEW_OK;
-        status = mlview_tree_editor2_get_cur_sel_start_iter 
+        status = mlview_tree_editor_get_cur_sel_start_iter 
                 (tree_editor, &cur_sel_start);
         g_return_val_if_fail (status == MLVIEW_OK, status) ;
-        mlview_tree_editor2_cut_node (tree_editor,
+        mlview_tree_editor_cut_node (tree_editor,
                                       &cur_sel_start);
         return MLVIEW_OK ;
 }
@@ -1094,7 +1097,7 @@ mlview_styled_view_cut_node (MlViewStyledView *a_this)
 enum MlViewStatus
 mlview_styled_view_copy_node (MlViewStyledView *a_this)
 {
-        MlViewTreeEditor2 *tree_editor = NULL;
+        MlViewTreeEditor *tree_editor = NULL;
         GtkTreeIter cur_sel_start = {0} ;
         enum MlViewStatus status = MLVIEW_OK ;
 
@@ -1106,10 +1109,10 @@ mlview_styled_view_copy_node (MlViewStyledView *a_this)
                 (a_this);
         if (tree_editor == NULL)
                 return MLVIEW_OK;
-        status = mlview_tree_editor2_get_cur_sel_start_iter
+        status = mlview_tree_editor_get_cur_sel_start_iter
                 (tree_editor, &cur_sel_start) ;
         g_return_val_if_fail (status == MLVIEW_OK, status) ;
-        mlview_tree_editor2_copy_node (tree_editor,
+        mlview_tree_editor_copy_node (tree_editor,
                                        &cur_sel_start) ;
         return MLVIEW_OK ;
 }
@@ -1135,12 +1138,12 @@ mlview_styled_view_paste_node_as_child (MlViewStyledView * a_this)
                               && PRIVATE (a_this)->tree_editor,
                               MLVIEW_BAD_PARAM_ERROR);
 
-        status = mlview_tree_editor2_get_cur_sel_start_iter 
+        status = mlview_tree_editor_get_cur_sel_start_iter 
                 (PRIVATE (a_this)->tree_editor,
                  &cur_sel_start) ;
         g_return_val_if_fail (status == MLVIEW_OK,
                               status) ;
-        mlview_tree_editor2_paste_node_as_child
+        mlview_tree_editor_paste_node_as_child
                 (PRIVATE (a_this)->tree_editor,
                  &cur_sel_start);
         return MLVIEW_OK ;
@@ -1165,11 +1168,11 @@ mlview_styled_view_paste_node_as_prev_sibling (MlViewStyledView * a_this)
                               && PRIVATE (a_this),
                               MLVIEW_BAD_PARAM_ERROR) ;
 
-        status = mlview_tree_editor2_get_cur_sel_start_iter
+        status = mlview_tree_editor_get_cur_sel_start_iter
                 (PRIVATE (a_this)->tree_editor, 
                  &cur_sel_start) ;
         g_return_val_if_fail (status == MLVIEW_OK, status) ;
-        status = mlview_tree_editor2_paste_node_as_sibling
+        status = mlview_tree_editor_paste_node_as_sibling
                 (PRIVATE (a_this)->tree_editor, 
                  &cur_sel_start, TRUE) ;
         return status ;
@@ -1190,11 +1193,11 @@ mlview_styled_view_paste_node_as_next_sibling (MlViewStyledView * a_this)
         g_return_val_if_fail (a_this != NULL, MLVIEW_BAD_PARAM_ERROR);
         g_return_val_if_fail (PRIVATE (a_this) != NULL, MLVIEW_BAD_PARAM_ERROR);
 
-        status = mlview_tree_editor2_get_cur_sel_start_iter
+        status = mlview_tree_editor_get_cur_sel_start_iter
                 (PRIVATE (a_this)->tree_editor,
                  &cur_sel_start) ;
         g_return_val_if_fail (status == MLVIEW_OK, status) ;
-        mlview_tree_editor2_paste_node_as_sibling
+        mlview_tree_editor_paste_node_as_sibling
                 (PRIVATE (a_this)->tree_editor,
                  &cur_sel_start, FALSE) ;
         return MLVIEW_OK ;
@@ -1262,7 +1265,7 @@ mlview_styled_view_connect_to_doc (MlViewIView *a_this,
         if (PRIVATE (view)->connected_to_doc == TRUE)
                 return MLVIEW_OK ;
 
-        mlview_tree_editor2_connect_to_doc (PRIVATE (view)->tree_editor,
+        mlview_tree_editor_connect_to_doc (PRIVATE (view)->tree_editor,
                                             a_doc) ;
         /*
          *TODO: connect the styled editor to the document signals
@@ -1303,7 +1306,7 @@ mlview_styled_view_disconnect_from_doc (MlViewIView *a_this,
         if (PRIVATE (view)->connected_to_doc == FALSE)
                 return MLVIEW_OK ;
 
-        mlview_tree_editor2_disconnect_from_doc (PRIVATE (view)->tree_editor, 
+        mlview_tree_editor_disconnect_from_doc (PRIVATE (view)->tree_editor, 
                                                  a_doc) ;
         /*
          *TODO: disconnect the styled editor to the document signals
@@ -1423,7 +1426,7 @@ mlview_styled_view_set_app_context (MlViewStyledView *a_this,
 }
 
 
-MlViewTreeEditor2 *
+MlViewTreeEditor *
 mlview_styled_view_get_tree_editor (MlViewStyledView *a_this)
 {
         g_return_val_if_fail (a_this && MLVIEW_IS_STYLED_VIEW (a_this),
