@@ -27,10 +27,12 @@
 #define __MLVIEW_XML_DOCUMENT_H__
 
 #include <libxml/tree.h>
+
 #include "mlview-app-context.h"
 #include "mlview-file-descriptor.h"
 #include "mlview-parsing-utils.h"
 #include "mlview-utils.h"
+#include "mlview-schema-list.h"
 
 G_BEGIN_DECLS
 /**
@@ -152,6 +154,15 @@ struct _MlViewXMLDocumentClass {
                                         xmlNs *a_ns,
                                         gpointer a_user_data) ;
 
+        void (*node_commented) (MlViewXMLDocument *a_this,
+                                xmlNode *a_node,
+                                xmlNode *a_new_node,
+                                gpointer a_user_data) ;
+
+        void (*node_uncommented) (MlViewXMLDocument *a_this,
+                                  xmlNode *a_node,
+                                  gpointer a_user_data) ;
+
         void (*node_changed) (MlViewXMLDocument *a_xml_doc,
                               xmlNode * a_node,
                               gpointer a_user_data);
@@ -195,10 +206,11 @@ struct _MlViewXMLDocumentClass {
                                               xmlEntity *a_entity_node,
                                               gpointer *a_user_data) ;
 
-        void (*dtd_associated) (MlViewXMLDocument *a_this,
-                                MlViewExtSubsDef *a_ext_subs_def,
-                                gpointer a_user_data);
+        void (*ext_subset_changed) (MlViewXMLDocument *a_this,
+                                    gpointer a_user_data);
 
+        void (*document_closed) (MlViewXMLDocument *a_this,
+                                 gpointer a_user_data);
 };
 
 guint mlview_xml_document_get_type (void);
@@ -212,14 +224,11 @@ void mlview_xml_document_ref (MlViewXMLDocument * a_this);
 
 void mlview_xml_document_unref (MlViewXMLDocument * a_this);
 
-MlViewXMLDocument *mlview_xml_document_open (gchar * a_file_name,
-                                             MlViewAppContext *a_context);
+MlViewXMLDocument *mlview_xml_document_open_with_dtd_interactive (gchar * a_file_name,
+                                                                  MlViewAppContext *a_context);
 
-MlViewXMLDocument *mlview_xml_document_open_with_dtd (gchar * a_file_name,
-                                                      gchar * a_dtd_name,
-                                                      MlViewAppContext * a_context);
+xmlNode * mlview_xml_document_get_node_from_clipboard2 (xmlDoc * a_xml_doc) ;
 
-xmlNode *mlview_xml_document_get_node_from_clipboard (xmlDoc * a_xml_doc);
 MlViewXMLDocument * mlview_xml_document_clone (MlViewXMLDocument *a_original);
 
 MlViewFileDescriptor * mlview_xml_document_get_file_descriptor (MlViewXMLDocument * a_xml_doc);
@@ -235,18 +244,10 @@ MlViewAppContext * mlview_xml_document_get_app_context (MlViewXMLDocument * a_do
 
 xmlDocPtr mlview_xml_document_get_xml_document (MlViewXMLDocument * a_xml_doc);
 
-void mlview_xml_document_set_xml_document (MlViewXMLDocument * a_doc,
-                                           xmlDocPtr a_mem_doc);
-
 void mlview_xml_document_set_app_context (MlViewXMLDocument * a_doc,
                                           MlViewAppContext * a_context);
 
-gint mlview_xml_document_associate_dtd_interactive (MlViewXMLDocument * a_doc);
-
-gint mlview_xml_document_associate_dtd (MlViewXMLDocument * a_doc,
-                                        MlViewExtSubsDef * a_ext_subs_def);
-
-gint mlview_xml_document_validate (MlViewXMLDocument * a_doc);
+void mlview_xml_document_validate (MlViewXMLDocument * a_doc);
 
 gboolean mlview_xml_document_needs_saving (MlViewXMLDocument * a_doc);
 
@@ -262,56 +263,55 @@ gint mlview_xml_document_save (MlViewXMLDocument *a_doc,
 void mlview_xml_document_select_node (MlViewXMLDocument *a_this,
                                       xmlNode *a_node) ;
 
-void mlview_xml_document_copy_node_to_clipboard  (xmlNode * a_xml_node, xmlDoc * a_doc);
+void mlview_xml_document_copy_node_to_clipboard2 (xmlNode * a_xml_node,
+                                                  xmlDoc * a_doc) ;
 
-xmlNode *mlview_xml_document_add_child_node (MlViewXMLDocument * a_xml_doc,
-                                             xmlNode * a_parent_xml_node,
-                                             xmlNode * a_xml_node,
-                                             gboolean a_subtree_required,
-                                             gboolean a_emit_signal);
+enum MlViewStatus mlview_xml_document_add_child_node (MlViewXMLDocument * a_xml_doc,
+                                                      const gchar * a_parent_xml_node_path,
+                                                      xmlNode * a_xml_node,
+                                                      gboolean a_subtree_required,
+                                                      gboolean a_emit_signal);
 
-xmlNode *mlview_xml_document_cut_node (MlViewXMLDocument * a_xml_doc,
-                                       xmlNode * a_xml_node, 
-                                       gboolean a_emit_signal);
+enum MlViewStatus mlview_xml_document_cut_node (MlViewXMLDocument * a_xml_doc,
+                                                const gchar * a_xml_node_path, 
+                                                gboolean a_emit_signal) ; 
 
-xmlNode *mlview_xml_document_insert_prev_sibling_node (MlViewXMLDocument * a_xml_document,
-                                                       xmlNode * a_sibling_node, 
-                                                       xmlNode * a_xml_node,
-                                                       gboolean a_subtree_required,
-                                                       gboolean a_emit_signal);
+enum MlViewStatus mlview_xml_document_insert_prev_sibling_node (MlViewXMLDocument * a_xml_document,
+                                                                const gchar* a_sibling_node_path, 
+                                                                xmlNode * a_xml_node,
+                                                                gboolean a_subtree_required,
+                                                                gboolean a_emit_signal);
 
-xmlNode *mlview_xml_document_insert_next_sibling_node (MlViewXMLDocument * a_xml_document,
-                                                       xmlNode * a_sibling_node, xmlNode * a_xml_node,
-                                                       gboolean a_subtree_required,
-                                                       gboolean a_emit_signal);
+enum MlViewStatus mlview_xml_document_insert_next_sibling_node (MlViewXMLDocument * a_xml_document,
+                                                                const gchar * a_sibling_node, 
+                                                                xmlNode * a_xml_node,
+                                                                gboolean a_subtree_required,
+                                                                gboolean a_emit_signal);
 
 void mlview_xml_document_paste_node_as_child (MlViewXMLDocument * a_xml_document,
-                                              xmlNode * a_parent_node,
+                                              const gchar * a_parent_node_path,
                                               gboolean a_emit_signal);
 
 void mlview_xml_document_paste_node_as_sibling (MlViewXMLDocument * a_xml_document,
-                                                xmlNode * a_parent_node,
-                                                xmlNode * a_sibling_node, 
+                                                const gchar * a_parent_node_path,
+                                                const gchar * a_sibling_node_path,
                                                 gboolean a_previous,
                                                 gboolean a_emit_signal);
 
 
-xmlNode *mlview_xml_document_set_node_content (MlViewXMLDocument * a_xml_document,
-                                               xmlNode * a_node, 
-                                               guchar * a_content,
-                                               enum MlViewEncoding a_enc,
-                                               gboolean a_emit_signal);
-
-
+enum MlViewStatus mlview_xml_document_set_node_content (MlViewXMLDocument * a_xml_document,
+                                                        const gchar * a_node_path,
+                                                        guchar * a_content,
+                                                        gboolean a_emit_signal) ;
+ 
 enum MlViewStatus mlview_xml_document_node_get_content  (xmlNode * a_node, 
                                                          enum MlViewEncoding a_enc,
                                                          guchar ** a_outbuf);
 
-xmlNode *mlview_xml_document_set_node_name (MlViewXMLDocument * a_xml_document,
-                                            xmlNode * a_node, 
-                                            guchar * a_name,
-                                            enum MlViewEncoding a_encoding,
-                                            gboolean a_emit_signal);
+enum MlViewStatus mlview_xml_document_set_node_name (MlViewXMLDocument * a_xml_document,
+                                                     const gchar * a_node_path,
+                                                     guchar * a_name,
+                                                     gboolean a_emit_signal);
 
 enum MlViewStatus mlview_xml_document_set_entity_node_name (MlViewXMLDocument *a_this,
                                                             xmlEntity *a_entity,
@@ -323,11 +323,11 @@ enum MlViewStatus mlview_xml_document_node_get_name (xmlNode * a_node,
                                                      enum MlViewEncoding a_enc, 
                                                      guchar ** a_outbuf);
 
-xmlAttr * mlview_xml_document_set_attribute (MlViewXMLDocument *a_this,
-                                             xmlNode *a_node,
-                                             const xmlChar* a_name,
-                                             const xmlChar *a_value,
-                                             gboolean a_emit_signal) ;
+enum MlViewStatus mlview_xml_document_set_attribute (MlViewXMLDocument *a_this,
+                                                     const gchar *a_node_path,
+                                                     const xmlChar* a_name,
+                                                     const xmlChar *a_value,
+                                                     gboolean a_emit_signal) ;
 
 enum MlViewStatus mlview_xml_document_set_attribute_name (MlViewXMLDocument *a_this,
                                                           xmlAttr *a_attr,
@@ -345,7 +345,7 @@ enum MlViewStatus mlview_xml_document_remove_attribute (MlViewXMLDocument *a_thi
                                                         gboolean a_emit_signal) ;
 
 enum MlViewStatus  mlview_xml_document_synch_attributes (MlViewXMLDocument *a_doc,
-                                                         xmlNode *a_node,
+                                                         const gchar *a_node_path,
                                                          GList *a_nv_pair_list) ;
 
 enum MlViewStatus mlview_xml_document_create_internal_subset (MlViewXMLDocument *a_this,
@@ -426,5 +426,49 @@ gboolean mlview_xml_document_is_completion_possible_global (MlViewXMLDocument *a
 gboolean mlview_xml_document_is_node_valid (MlViewXMLDocument *a_this,
                                             xmlNode *a_node);
 
+const gchar *mlview_xml_document_get_ext_subset_name (MlViewXMLDocument *a_doc);
+
+MlViewSchemaList *mlview_xml_document_get_schema_list (MlViewXMLDocument *a_doc);
+
+gboolean mlview_xml_document_set_ext_subset_with_url (MlViewXMLDocument *a_doc,
+                                                      const gchar *a_url);
+
+MlViewXMLDocument *mlview_xml_document_open_with_dtd (gchar * a_file_name,
+                                                      gchar * a_dtd_name,
+                                                      MlViewAppContext * a_context);
+
+
+enum MlViewStatus mlview_xml_document_replace_node (MlViewXMLDocument *a_this,
+                                                    xmlNode *a_node,
+                                                    xmlNode *a_replacement,
+                                                    gboolean a_emit_signal) ;
+
+enum MlViewStatus mlview_xml_document_comment_node (MlViewXMLDocument *a_this,
+                                                    xmlNode *a_node,
+                                                    gboolean a_emit_signal) ;
+
+
+gchar *mlview_xml_document_get_uri (MlViewXMLDocument *a_this);
+
+gboolean mlview_xml_document_can_undo_mutation (MlViewXMLDocument *a_this) ;
+
+gboolean mlview_xml_document_can_redo_mutation (MlViewXMLDocument *a_this) ;
+
+enum MlViewStatus mlview_xml_document_undo_mutation (MlViewXMLDocument *a_this,
+                                                     gpointer a_user_data) ;
+
+enum MlViewStatus mlview_xml_document_redo_mutation (MlViewXMLDocument *a_this,
+                                                     gpointer a_user_data) ;
+
+xmlNode * mlview_xml_document_get_node_from_xpath (MlViewXMLDocument *a_this,
+                                                   const gchar *an_xpath_expr) ;
+
+enum MlViewStatus mlview_xml_document_get_node_path (MlViewXMLDocument *a_this,
+                                                     xmlNode *a_node,
+                                                     gchar **a_path) ;
+
+enum MlViewStatus mlview_xml_document_get_root_element (MlViewXMLDocument *a_this,
+                                                        xmlNode **a_root_element) ;
+
 G_END_DECLS
 #endif
